<?php

namespace Odhen\API\Service;

use Odhen\API\Remote\Printer\Command;

class ImpressaoPedidoSmart {

    protected $entityManager;
    protected $impressaoUtil;
    protected $instanceManager;
	protected $util;
    public $logos = [];

    public function __construct(
        \Doctrine\ORM\EntityManager $entityManager,
        \Odhen\API\Lib\ImpressaoUtil $impressaoUtil,
        \Zeedhi\Framework\DependencyInjection\InstanceManager $instanceManager,
        \Odhen\API\Util\Util $util
    ) {
        $this->entityManager = $entityManager;
        $this->impressaoUtil = $impressaoUtil;
        $this->comandos = array();
        $this->instanceManager = $instanceManager;
        $this->util 	     = $util;
    }

    public function imprimePedidoSmart($CDFILIAL, $CDLOJA, $produtos, $CDVENDEDOR, $DSCOMANDA, $NRMESA, $CDSENHAPED,
                                  $tipoVenda, $toGoString, $multiplasComandas, $ultimaComanda, $CDCAIXA, $NRNOTAFISCALCE, $NRORG, $reimpressao = null){
        
        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'CDLOJA' => $CDLOJA
        );
        $dadosLoja = $this->entityManager->getConnection()->fetchAll("BUSCA_DADOS_LOJA", $params);
        $IDLUGARMESA = $dadosLoja[0]['IDLUGARMESA'];
        $IDPOSOBSPED = $dadosLoja[0]['IDPOSOBSPED'];
        $IDUTLCORTEPED = $dadosLoja[0]['IDUTLCORTEPED'];

        // TipoVenda B (balcão) do TAA não tem mesa.
        if (!empty($NRMESA)) {
            $params = array(
                'CDFILIAL' => $CDFILIAL,
                'CDLOJA' => $CDLOJA,
                'NRMESA' => $NRMESA
            );
            $dadosMesa = $this->entityManager->getConnection()->fetchAll("BUSCA_DADOS_MESA", $params);
            $NMSALA = $dadosMesa[0]['NMSALA'];
        }
        else $NMSALA = null;

        $params = array(
            'CDVENDEDOR' => $CDVENDEDOR
        );
        $dadosVendedor = $this->entityManager->getConnection()->fetchAll("BUSCA_DADOS_VENDEDOR", $params);

        if (!empty($dadosVendedor[0]['NMFANVEN'])) $NMFANVEN = $dadosVendedor[0]['NMFANVEN'];
        else $NMFANVEN = "";

        try {
            $NMEVENTO = "";
            $DTINICIALEVENTO = "";
            $DSURLLOGO = "";
            $params = array(
                'CDFILIAL' => $CDFILIAL,
                'CDCAIXA' => $CDCAIXA,
                'NRORG' => $NRORG
            );
            $dadosEvento = $this->entityManager->getConnection()->fetchAll("BUSCA_DADOS_EVENTO", $params);
            if(!empty($dadosEvento)){
                foreach ($dadosEvento as $key => $value) {
                    $NMEVENTO = $value['NMEVENTO'];
                    $HRINICIAL = '';
                    $DTINICIAL = '';
                    /*if(!empty($value['HRINICIAL'])){
                        $HRINICIAL = ' '.$value['HRINICIAL'];
                    }*/
                    if(!empty($value['DTINICIAL'])){
                        $DTINICIAL = $value['DTINICIAL'].' ';
                    }
                    $DTINICIALEVENTO = $DTINICIAL.$HRINICIAL;
                    if(isset($value['DSURLLOGO'])){
                        if(!empty($value['DSURLLOGO'])){
                            $arrTemp = $this->entityManager->getConnection()->fetchAssoc("BUSCA_FILESERVER");
                            if($arrTemp){
                                if(!empty($arrTemp['FILESERVERURL'])){
                                    $DSURLLOGO = $arrTemp['FILESERVERURL'].$value['DSURLLOGO'];
                                }
                            }
                        }
                    }
                    break;
                }
            }
        } catch (\Exception $e) { }

        /*$NMEVENTO = "JOGO CAM X FLU 25.09.2024";
        $DTINICIALEVENTO = "00/00/0000 00:00";
        $DSURLLOGO = "https://midia.teknisa.com/files/Manage588fa3b08b544/Eventos/66f6e1b028f9c.png";*/

        // Verifica parametro IDIMPRODUVEZ dos produtos para replicá-los para impressão.
        $produtos = $this->separaProdImpUmPorVez($produtos);
        // Reorganiza o array para que os produtos que serão impressos separados (IDIMPRODUVEZ) fiquem no final.
        $produtos = $this->alteraOrdemImpressao($produtos);

        $produtosIndexadosPorImpressora = self::indexaProdutosPorImpressora($CDFILIAL, $CDLOJA, $produtos);
        $impressoraProducao = $produtosIndexadosPorImpressora['impressoraProducao'];
        $impressoraProducao2 = $produtosIndexadosPorImpressora['impressoraProducao2'];
        $impressoraPuxa = $produtosIndexadosPorImpressora['impressoraPuxa'];
        $result = array(
            'error' => false
        );
        foreach ($impressoraProducao as $impressoraAtual){
            $cutByPosition = $IDUTLCORTEPED === 'S';
            $respostaPedido = self::montaPedido($impressoraAtual, $DSCOMANDA, $NRMESA, $NMSALA, $CDVENDEDOR,
             $NMFANVEN, $CDSENHAPED, $IDLUGARMESA, $IDPOSOBSPED, $tipoVenda, $toGoString, $cutByPosition, $multiplasComandas, $ultimaComanda,
             $NMEVENTO, $DTINICIALEVENTO, $DSURLLOGO, $NRNOTAFISCALCE, $reimpressao);
            if ($respostaPedido['error']) $result = $respostaPedido;
        }
        foreach ($impressoraProducao2 as $impressoraAtual){
            $cutByPosition = $IDUTLCORTEPED === 'S';
            $respostaPedido = self::montaPedido($impressoraAtual, $DSCOMANDA, $NRMESA, $NMSALA, $CDVENDEDOR,
             $NMFANVEN, $CDSENHAPED, $IDLUGARMESA, $IDPOSOBSPED, $tipoVenda, $toGoString, $cutByPosition, $multiplasComandas, $ultimaComanda,
             $NMEVENTO, $DTINICIALEVENTO, $DSURLLOGO, $NRNOTAFISCALCE, $reimpressao);
            if ($respostaPedido['error']) $result = $respostaPedido;
        }
        foreach ($impressoraPuxa as $impressoraAtual){
            $respostaPedido = self::montaPedido($impressoraAtual, $DSCOMANDA, $NRMESA, $NMSALA, $CDVENDEDOR,
             $NMFANVEN, $CDSENHAPED, $IDLUGARMESA, $IDPOSOBSPED, $tipoVenda, $toGoString, false, $multiplasComandas, $ultimaComanda,
             $NMEVENTO, $DTINICIALEVENTO, $DSURLLOGO, $NRNOTAFISCALCE, $reimpressao);
            if ($respostaPedido['error']) $result = $respostaPedido;
        }

        if($ultimaComanda){
            $dadosImpressora = $this->impressaoUtil->getDadosImpressora($CDFILIAL, $CDCAIXA);
            $IDUTILIMPFRONT = $dadosImpressora['dadosImpressora']['IDUTILIMPFRONT'];
            $issaas = $this->util->isSaas();
            $result['paramsImpressora'] = [];
            foreach ($this->comandos as $comando) {
                array_push($result['paramsImpressora'],
                    array(
                            'saas'      => false,
                            'IDUTILIMPFRONT' => ($IDUTILIMPFRONT=='S'),
                            'impressora'=> $comando['impressora'],
                            'comandos'  => $comando['comandos']->getCommands(),
                            'error'     => false
                    )
                );
                $respostaPonte['error'] = false;
            }
            $this->comandos = array();
        }
        return $result;
    }

    private function montaPedido($impressoraAtual, $DSCOMANDA, $NRMESA, $NMSALA, $CDVENDEDOR, $NMFANVEN, $CDSENHAPED, $IDLUGARMESA, $IDPOSOBSPED, $tipoVenda, $toGoString, $cutByPosition, $multiplasComandas, $ultimaComanda, $NMEVENTO, $DTEVENTO, $DSURLLOGO, $NRNOTAFISCALCE, $reimpressao){

        $posicoes = array_unique(array_column($impressoraAtual['produtos'], 'NRLUGARMESA'));

        if(!isset($this->comandos[$impressoraAtual['NRSEQIMPRLOJA']])){
            $this->comandos[$impressoraAtual['NRSEQIMPRLOJA']]['comandos'] = new Command();
            $this->comandos[$impressoraAtual['NRSEQIMPRLOJA']]['impressora'] = $impressoraAtual;
        }

        if($cutByPosition){
            foreach ($posicoes as $posicao) {
                $retornoImprime =  $this->_imprimePedido(
                    $impressoraAtual, $DSCOMANDA, $NRMESA, $NMSALA, $CDVENDEDOR, $NMFANVEN, $CDSENHAPED,
                    $IDLUGARMESA, $IDPOSOBSPED, $tipoVenda, $toGoString, array($posicao),
                    $this->comandos[$impressoraAtual['NRSEQIMPRLOJA']]['comandos'], $multiplasComandas,
                    $NMEVENTO, $DTEVENTO, $DSURLLOGO, $NRNOTAFISCALCE, $reimpressao
                );
                if($retornoImprime['error']){
                    return $retornoImprime;
                }
            }
        } else {
            $retornoImprime = $this->_imprimePedido(
                $impressoraAtual, $DSCOMANDA, $NRMESA, $NMSALA, $CDVENDEDOR, $NMFANVEN, $CDSENHAPED,
                $IDLUGARMESA, $IDPOSOBSPED, $tipoVenda, $toGoString, $posicoes,
                $this->comandos[$impressoraAtual['NRSEQIMPRLOJA']]['comandos'], $multiplasComandas,
                $NMEVENTO, $DTEVENTO, $DSURLLOGO, $NRNOTAFISCALCE, $reimpressao
            );
        }
        if($retornoImprime['error']){
            return $retornoImprime;
        }else{
            if($multiplasComandas){
                if($ultimaComanda){
                    foreach ($this->comandos as $comando) {
                        $comando['comandos']->cutPaper();
                    }
                }
                return array('error' => false);
            }else{
                return array('error' => false);
            }
        }
    }

    public function addLogo($comandos, $printerParams, $DSURLLOGO){
        try {
            //baixa a logo e salva a logo para que caso tenha mais de um produto na mesma venda não precisar baixar novamente.
            if(!empty($DSURLLOGO)){
                if(isset($this->logos[$DSURLLOGO])){
                    //envia somente parte da string porque a primeira já foi enviada completa
                    $comandos->imagemBase64(substr($this->logos[$DSURLLOGO],0,50), $printerParams);
                } else {
                    $imageContent = file_get_contents($DSURLLOGO);
                    if ($imageContent !== false) {
                        $base64Image = base64_encode($imageContent);
                        if(!empty($base64Image)){
                            $this->logos[$DSURLLOGO] = $base64Image;
                            $comandos->imagemBase64($base64Image, $printerParams);
                        }
                    }
                }                
            }
        } catch (\Exception $e) {
            //var_dump($e->getMessage());
        }
    }

    private function _imprimePedido($impressoraAtual, $DSCOMANDA, $NRMESA, $NMSALA, $CDVENDEDOR, $NMFANVEN, $CDSENHAPED, $IDLUGARMESA, $IDPOSOBSPED, $tipoVenda, $toGoString, $posicoes, $comandos, $multiplasComandas, $NMEVENTO, $DTEVENTO, $DSURLLOGO, $NRNOTAFISCALCE, $reimpressao) {
        $produtos = $impressoraAtual['produtos'];
        $tipoImpressao = $impressoraAtual['tipoImpressao'];

        $printerParams = $this->impressaoUtil->buscaParametrosImpressora($impressoraAtual['IDMODEIMPRES']);
        $printerParams['letterType'] = $printerParams['tipoLetra'];
        $printerParams['bold'] = false;
        $printerParams['expanded'] = false;
        $stEnter = $printerParams['comandoEnter'];
        if($impressoraAtual['IDMODEIMPRES'] == '40') {
            $printerParams['largura'] = 43;
            $printerParams['larguraCupom'] = 43;
        }

        //da pra otimizar
        $arrNmPosicao = $this->buscaNomePorPosicao($produtos, $posicoes);

        $textHeader = $this->printHeader(
            "", $printerParams, $tipoVenda, $DSCOMANDA, $NRMESA, $NMSALA,
            $CDVENDEDOR, $NMFANVEN, $tipoImpressao, $toGoString, $comandos, $reimpressao
        );

        //logo
        foreach ($produtos as $key => $produtoAtual){
            //valida se a primeira posição é imprime 1 por vez
            if ($produtoAtual['IDIMPRODUVEZ'] == 'S'){
                self::addLogo($comandos, $printerParams, $DSURLLOGO);
            }
            break;
        }

        // header
        $comandos->text($textHeader, $printerParams);

        // footer.
        $textFooter = $this->impressaoUtil->replicate(' ', $printerParams['largura'] ). $stEnter;
        $textFooter .= $this->printFooter("", $printerParams, $CDSENHAPED, $NRNOTAFISCALCE) . $stEnter . $stEnter . $stEnter;
        $textFooter .= $this->impressaoUtil->replicate('-', $printerParams['largura'] ). $stEnter;
        $textFooter .= $this->impressaoUtil->replicate(' ', $printerParams['largura'] ). $stEnter;

        foreach ($posicoes as $posicao) {
            $printerParams['bold'] = false;
            if ($IDLUGARMESA == 'S' && $tipoVenda == 'M') {
                //imprime a posição no pedido
                $strPosicao = 'POSICAO ' . $posicao;
                if (isset($arrNmPosicao[$posicao])){
                    $strPosicao .= ' - ' . $arrNmPosicao[$posicao]['DSCONSUMIDOR'];
                    if (strlen($strPosicao) > $printerParams['largura']) {
                        $strPosicao = $this->impressaoUtil->quebraLinha($strPosicao, $printerParams, true);
                    }
                }
                $comandos->text($strPosicao . $stEnter, $printerParams);
            }

            $totaldeProdutos = 0;
            foreach ($produtos as $key => $produtoAtual){
                if ( $posicao == $produtoAtual['NRLUGARMESA'] && ( !empty($produtoAtual['CDPRODPROMOCAO']) || (empty($produtoAtual['CDPRODPROMOCAO']) && $produtoAtual['IDIMPPRGRPROMO'] == 'N') ) ){
                    if ($produtoAtual['IDIMPRODUVEZ'] == 'S'){
                        $totaldeProdutos = $totaldeProdutos + 1;
                    }
                }
            }
            $contador = 0;
            foreach ($produtos as $key => $produtoAtual){
                $produtos[$key]['TOTALIZACAO'] = '';
                if ( $posicao == $produtoAtual['NRLUGARMESA'] && ( !empty($produtoAtual['CDPRODPROMOCAO']) || (empty($produtoAtual['CDPRODPROMOCAO']) && $produtoAtual['IDIMPPRGRPROMO'] == 'N') ) ){
                    if ($produtoAtual['IDIMPRODUVEZ'] == 'S'){
                        $contador = $contador+1;
                        $produtos[$key]['TOTALIZACAO'] = $contador.'/'.$totaldeProdutos;
                    }
                }
            }
            $ordemImp = 0;
            foreach ($produtos as $key => $produtoAtual){
                //imprime produto
                if ( $posicao == $produtoAtual['NRLUGARMESA'] && ( !empty($produtoAtual['CDPRODPROMOCAO']) || (empty($produtoAtual['CDPRODPROMOCAO']) && $produtoAtual['IDIMPPRGRPROMO'] == 'N') ) ){
                    if (!empty($produtoAtual['ORDEMIMP']) && ($produtoAtual['ORDEMIMP'] - $ordemImp >= 1) && ($ordemImp != 0)) {
                        $ordemImp = 0;
                    }

                    if ($produtoAtual['IDIMPRODUVEZ'] == 'S'){
                        // Caso o pedido seja impresso separado, imprime o rodapé e cabeçalho novamente.
                        if ($key !== 0 ){
                            $comandos->text($textFooter, $printerParams);
                            $comandos->text($this->impressaoUtil->replicate(' ', $printerParams['largura'] ). $stEnter, $printerParams);
                            self::addLogo($comandos, $printerParams, $DSURLLOGO);
                            $comandos->text($textHeader, $printerParams);
                        }
                    }
                    if ($produtoAtual['IDIMPPROMOCAO'] !== 'S') {
                        if ($produtoAtual['IDIMPRODUVEZ'] == 'S'){
                            if(!empty($NMEVENTO) && !empty($DTEVENTO)){
                                $printerParams['bold'] = true;
                                $printerParams['expanded'] = false;
                                $comandos->text($this->impressaoUtil->centraliza($printerParams, $NMEVENTO, $printerParams['largura']), $printerParams);
                                $printerParams['bold'] = false;
                                $printerParams['expanded'] = false;
                                $comandos->text($this->impressaoUtil->centraliza($printerParams, "Valido ate ".$DTEVENTO, $printerParams['largura']). $stEnter, $printerParams);
                                $comandos->text($this->impressaoUtil->replicate(' ', $printerParams['largura'] ). $stEnter, $printerParams);
                            }
                        }
                    }

                    if ($produtoAtual['IDIMPPROMOCAO'] == 'S') { // Cabeçalho da promoção inteligente.
                        $comandos->text( $this->impressaoUtil->centraliza($printerParams, $produtoAtual['NMPRODPROMOCAO']) . $stEnter, $printerParams);
                        if (!empty($produtoAtual['TXPRODCOMVEN'])){
                            // Imprime observação do produto pai.
                            $comandos->text( $this->impressaoUtil->centraliza($printerParams, '* ' . $produtoAtual['TXPRODCOMVEN'] . $stEnter), $printerParams);
                            $produtoAtual['TXPRODCOMVEN'] = null;
                        }
                        // Variável usada para imprimir a última linha da promoção inteligente.
                        $ordemImp = bcdiv(str_replace(',','.',strval($produtoAtual['ORDEMIMP'])), '1', '0');
                    }

                    if ($IDPOSOBSPED == 'A'){// Imprimir observações antes do produto.
                        if ( !empty(str_replace(" ","",$produtoAtual['TXPRODCOMVEN'])) ){
                            $comandos->text( '   *  ' . $produtoAtual['TXPRODCOMVEN'] . $stEnter, $printerParams);
                        }
                        if ($produtoAtual['ATRASOPROD']){
                            $comandos->text( '   *  Segura de ' . $produtoAtual['ATRASOPROD'] . ' minutos' . $stEnter, $printerParams);
                        }
                    }

                    if ($produtoAtual['IDIMPPROMOCAO'] !== 'S') { // Imprime a linha do produto, exceto se for cabeçalho da promoção.
                        $printerParams['bold'] = true;
                        if ($produtoAtual['IDIMPRODUVEZ'] == 'S'){
                            if(isset($produtoAtual['NMPRODPROMOCAO'])){
                                if(!empty($produtoAtual['NMPRODPROMOCAO']) && $produtoAtual['IDTIPOCOMPPROD']=='1' && $produtoAtual['IDIMPPROMOCAO']=='X'){
                                    $printerParams['expanded'] = false;
                                    $comandos->text($this->impressaoUtil->centraliza($printerParams, $produtoAtual['NMPRODPROMOCAO'], $printerParams['largura']). $stEnter, $printerParams);
                                }
                            }
                            $printerParams['expanded'] = true;
                        }
                        $nomeProduto = $produtoAtual['QTPRODCOMVEN'] . ' ' . $this->util->removeAcentos($produtoAtual['NMPRODUTO']);
                        $comandos->text($nomeProduto . $stEnter, $printerParams);
                        $printerParams['expanded'] = false;
                        $printerParams['bold'] = false;
                        if ($produtoAtual['IDIMPRODUVEZ'] == 'S'){
                            if(isset($produtoAtual['VRUNITVEND'])){
                                if(!empty($produtoAtual['VRUNITVEND'])){
                                    $valordesconto = 0;
                                    if($produtoAtual['VRDESITVEND']>0){
                                        $valordesconto = floatval($produtoAtual['VRDESITVEND']);
                                    }
                                    $valorItem = $this->impressaoUtil->formataNumero(floatval($produtoAtual['VRUNITVEND']-$valordesconto), 2);
                                    $printerParams['bold'] = true;
                                    $printerParams['expanded'] = true;
                                    $comandos->text($stEnter . ' R$'.$valorItem . $stEnter, $printerParams);
                                    $printerParams['bold'] = false;
                                    $printerParams['expanded'] = false;
                                }
                            }
                        }
                    }

                    if ($IDPOSOBSPED == 'D'){ // Imprime observação depois do produto.
                        if ( !empty(str_replace(" ","",$produtoAtual['TXPRODCOMVEN'])) ){
                            $comandos->text( '   *  ' . $produtoAtual['TXPRODCOMVEN'] . $stEnter, $printerParams);
                        }
                        if ($produtoAtual['ATRASOPROD']){
                            $comandos->text( '   *  Segura de ' . $produtoAtual['ATRASOPROD'] . ' minutos' . $stEnter, $printerParams);
                        }
                    }

                    if (!empty($produtoAtual['IDORIGEMVENDA'])) { // Produto para viagem.
                        $comandos->text('   *  Para viagem ' . $stEnter, $printerParams);
                    }

                    $printerParams['bold'] = false;
                    $printerParams['expanded'] = false;
                    if ($produtoAtual['IDIMPRODUVEZ'] == 'S'){
                        if(isset($produtoAtual['TOTALIZACAO'])){
                            if(!empty($produtoAtual['TOTALIZACAO'])){
                                $comandos->text($this->impressaoUtil->centraliza($printerParams, '  '.$produtoAtual['TOTALIZACAO'].'  ' , $printerParams['largura']). $stEnter, $printerParams);
                            }
                        }
                    }

                    if ($produtoAtual['IDIMPRODUVEZ'] === 'S'){
                        $splitPrint = false;
                    }

                    //tira do array pra nao precisar checar na proxima observacao
                    unset($produtos[$key]);                 
                }
            }
        }

        $printerParams['bold'] = false;
        $comandos->text($textFooter, $printerParams);
        $comandos->text($this->impressaoUtil->replicate(' ', $printerParams['largura'] ). $stEnter, $printerParams);

        return array('error' => false);
    }

    private function indexaProdutosPorImpressora($CDFILIAL, $CDLOJA, $produtos) {
        $impressoraProducao = array();
        $impressoraProducao2 = array();
        $impressoraPuxa = array();
        foreach ($produtos as $produtoAtual) {
            if (empty($produtoAtual['ORDEM'])) {
                $produtoAtual['ORDEM'] = 0;
            }
            if ($produtoAtual['IDTIPOCOMPPROD'] == '3') {
                $produtoAtual['IDIMPPROMOCAO'] = 'S';
                $params = array(
                    'CDFILIAL' => $CDFILIAL,
                    'CDLOJA' => $CDLOJA,
                    'CDPRODUTO' => $produtoAtual['CDPRODUTO']
                );
                $impressoras = $this->entityManager->getConnection()->fetchAll("BUSCA_IMPRESSORAS", $params);
                self::buscaImpressoraPorProduto($impressoras, $produtoAtual, $CDFILIAL, $CDLOJA, $impressoraProducao, $impressoraProducao2, $impressoraPuxa);
            } else {
                $params = array(
                    'CDFILIAL' => $CDFILIAL,
                    'CDLOJA' => $CDLOJA,
                    'CDPRODUTO' => $produtoAtual['CDPRODUTO']
                );
                $impressoras = $this->entityManager->getConnection()->fetchAll("BUSCA_IMPRESSORAS", $params);
                self::buscaImpressoraPorProduto($impressoras, $produtoAtual, $CDFILIAL, $CDLOJA, $impressoraProducao, $impressoraProducao2, $impressoraPuxa);
            }
        }
        return array(
            'impressoraProducao' => $impressoraProducao,
            'impressoraProducao2' => $impressoraProducao2,
            'impressoraPuxa' => $impressoraPuxa
        );
    }

    private function buscaImpressoraPorAmbiente($CDFILIAL, $CDLOJA, $produtoAtual) {
    	$impressoraPorAmbiente = array();
    	if (!empty($produtoAtual['CDSALA']) && empty($produtoAtual['NRSEQIMPRLOJA'])) {
			$params = array(
                'CDFILIAL' => $CDFILIAL,
                'CDLOJA' => $CDLOJA,
                'CDPRODUTO' => $produtoAtual['CDPRODUTO'],
                'NRCONFTELA' => $produtoAtual['NRCONFTELA'],
                'DTINIVIGENCIA' => new \DateTime($produtoAtual['DTINIVIGENCIA']),
                'CDAMBIENTE' => $produtoAtual['CDSALA']
            );
            $types = array(
                'DTINIVIGENCIA' => \Doctrine\DBAL\Types\Type::DATETIME
            );
            $retornoQuery = $this->entityManager->getConnection()->fetchAll("BUSCA_IMPRESSORAS_POR_AMBIENTE", $params, $types);
            if (sizeof($retornoQuery) === 1) {
            	$impressoraPorAmbiente = $retornoQuery;
            }
    	}
    	return $impressoraPorAmbiente;
    }

    private function buscaImpressoraPorProduto($impressoras, $produtoAtual, $CDFILIAL, $CDLOJA, &$impressoraProducao, &$impressoraProducao2, &$impressoraPuxa){
        if (!empty($produtoAtual['NRSEQIMPRLOJA'])) {
            $params = array(
                'CDFILIAL' => $CDFILIAL,
                'CDLOJA' => $CDLOJA,
                'NRSEQIMPRLOJA' => $produtoAtual['NRSEQIMPRLOJA']
            );
            $dadosImpressora = $this->entityManager->getConnection()->fetchAll("BUSCA_DADOS_IMPRESSORA_PED", $params);
            if (!empty($impressoraProducao[$dadosImpressora[0]['NRSEQIMPRLOJA']])) {
                array_push($impressoraProducao[$dadosImpressora[0]['NRSEQIMPRLOJA']]['produtos'], $produtoAtual);
            } else {
                $impressoraProducao[$dadosImpressora[0]['NRSEQIMPRLOJA']] = array(
                    'NRSEQIMPRLOJA' => $dadosImpressora[0]['NRSEQIMPRLOJA'],
                    'IDMODEIMPRES' => $dadosImpressora[0]['IDMODEIMPRES'],
                    'tipoImpressao' => 'P',
                    'CDPORTAIMPR' => $dadosImpressora[0]['CDPORTAIMPR'],
                    'DSENDPORTA' => $dadosImpressora[0]['DSENDPORTA'],
                    'DSIPIMPR' => $dadosImpressora[0]['DSIPIMPR'],
                    'DSIPPONTE' => $dadosImpressora[0]['DSIPPONTE'],
                    'NMIMPRLOJA' => $dadosImpressora[0]['NMIMPRLOJA'],
                    'produtos' => array($produtoAtual)
                );
            }
        } else {
        	$impressoraPorAmbiente = $this->buscaImpressoraPorAmbiente($CDFILIAL, $CDLOJA, $produtoAtual);
        	if (!empty($impressoraPorAmbiente[0]['NRSEQIMPRLOJA'])) {
        		if (!empty($impressoraProducao[$impressoraPorAmbiente[0]['NRSEQIMPRLOJA']])) {
	                array_push($impressoraProducao[$impressoraPorAmbiente[0]['NRSEQIMPRLOJA']]['produtos'], $produtoAtual);
	            } else {
	                $impressoraProducao[$impressoraPorAmbiente[0]['NRSEQIMPRLOJA']] = array(
	                    'NRSEQIMPRLOJA' => $impressoraPorAmbiente[0]['NRSEQIMPRLOJA'],
	                    'IDMODEIMPRES' => $impressoraPorAmbiente[0]['IDMODEIMPRES'],
	                    'tipoImpressao' => 'P',
	                    'CDPORTAIMPR' => $impressoraPorAmbiente[0]['CDPORTAIMPR'],
	                    'DSENDPORTA' => $impressoraPorAmbiente[0]['DSENDPORTA'],
	                    'DSIPIMPR' => $impressoraPorAmbiente[0]['DSIPIMPR'],
	                    'DSIPPONTE' => $impressoraPorAmbiente[0]['DSIPPONTE'],
                        'NMIMPRLOJA' => $impressoraPorAmbiente[0]['NMIMPRLOJA'],
	                    'produtos' => array($produtoAtual)
	                );
	            }
	        } else if (!empty($impressoras[0]['A_NRSEQIMPRPROD'])) {
	            if (!empty($impressoraProducao[$impressoras[0]['A_NRSEQIMPRPROD']])) {
	                array_push($impressoraProducao[$impressoras[0]['A_NRSEQIMPRPROD']]['produtos'], $produtoAtual);
	            } else {
	                $impressoraProducao[$impressoras[0]['A_NRSEQIMPRPROD']] = array(
	                    'NRSEQIMPRLOJA' => $impressoras[0]['A_NRSEQIMPRPROD'],
	                    'IDMODEIMPRES' => $impressoras[0]['A_IDMODEIMPRES'],
	                    'tipoImpressao' => 'P',
	                    'CDPORTAIMPR' => $impressoras[0]['A_CDPORTAIMPR'],
	                    'DSENDPORTA' => $impressoras[0]['A_DSENDPORTA'],
	                    'DSIPIMPR' => $impressoras[0]['A_DSIPIMPR'],
	                    'DSIPPONTE' => $impressoras[0]['A_DSIPPONTE'],
                        'NMIMPRLOJA' => $impressoras[0]['A_NMIMPRLOJA'],
	                    'produtos' => array($produtoAtual)
	                );
	            }
	        }
	    }

        if (!empty($impressoras[0]['B_NRSEQIMPRPROD2'])) {
            if (!empty($impressoraProducao2[$impressoras[0]['B_NRSEQIMPRPROD2']])) {
                array_push($impressoraProducao2[$impressoras[0]['B_NRSEQIMPRPROD2']]['produtos'], $produtoAtual);
            } else {
                $impressoraProducao2[$impressoras[0]['B_NRSEQIMPRPROD2']] = array(
                    'NRSEQIMPRLOJA' => $impressoras[0]['B_NRSEQIMPRPROD2'],
                    'IDMODEIMPRES' => $impressoras[0]['B_IDMODEIMPRES'],
                    'tipoImpressao' => 'P',
                    'CDPORTAIMPR' => $impressoras[0]['B_CDPORTAIMPR'],
                    'DSENDPORTA' => $impressoras[0]['B_DSENDPORTA'],
                    'DSIPIMPR' => $impressoras[0]['B_DSIPIMPR'],
                    'DSIPPONTE' => $impressoras[0]['B_DSIPPONTE'],
                    'NMIMPRLOJA' => $impressoras[0]['B_NMIMPRLOJA'],
                    'produtos' => array($produtoAtual)
                );
            }
        }

        if (!empty($impressoras[0]['C_NRSEQIMPRPUXA'])) {
            if (!empty($impressoraPuxa[$impressoras[0]['C_NRSEQIMPRPUXA']])) {
                array_push($impressoraPuxa[$impressoras[0]['C_NRSEQIMPRPUXA']]['produtos'], $produtoAtual);
            } else {
                $impressoraPuxa[$impressoras[0]['C_NRSEQIMPRPUXA']] = array(
                    'NRSEQIMPRLOJA' => $impressoras[0]['C_NRSEQIMPRPUXA'],
                    'IDMODEIMPRES' => $impressoras[0]['C_IDMODEIMPRES'],
                    'tipoImpressao' => 'U',
                    'CDPORTAIMPR' => $impressoras[0]['C_CDPORTAIMPR'],
                    'DSENDPORTA' => $impressoras[0]['C_DSENDPORTA'],
                    'DSIPIMPR' => $impressoras[0]['C_DSIPIMPR'],
                    'DSIPPONTE' => $impressoras[0]['C_DSIPPONTE'],
                    'NMIMPRLOJA' => $impressoras[0]['C_NMIMPRLOJA'],
                    'produtos' => array($produtoAtual)
                );
            }
        }
        return array(
            'impressoraProducao' => $impressoraProducao,
            'impressoraProducao2' => $impressoraProducao2,
            'impressoraPuxa' => $impressoraPuxa
        );
    }

    private function printHeader($text, $printerParams, $tipoVenda, $DSCOMANDA, $NRMESA,
                                 $NMSALA, $CDVENDEDOR, $NMFANVEN, $tipoImpressao, $toGoString, $comandos, $reimpressao) {
        // Monta o cabeçalho do pedido.
        $title = $tipoImpressao == 'P' ? 'PEDIDO' : 'RELATORIO DO PUXA';
        $title .= $reimpressao == 'S' ? ' - REIMPRESSÃO' : '';

        $text .= $printerParams['comandoEnter'];
        $text .= $this->impressaoUtil->centraliza($printerParams, $title) . $printerParams['comandoEnter'];

        $data = new \DateTime('NOW', new \DateTimeZone('America/Bahia'));

        $text .= $this->impressaoUtil->centraliza($printerParams, 'DATA: ' . $data->format('d/m/Y H:i:s')) . $printerParams['comandoEnter'];

        if (!empty($NRMESA)) {
            $printerParams['bold'] = false;
            $comandos->text($text, $printerParams);
            $text = '';

            $text .= $this->impressaoUtil->centraliza($printerParams, 'MESA '. $NRMESA, $printerParams['largura']) . $printerParams['comandoEnter'];

            $printerParams['bold'] = true;
            $comandos->text($text, $printerParams);
            $text = '';
        }
        if ($tipoVenda == 'C') {
            $text .= 'COMANDA.: ' . $DSCOMANDA . $printerParams['comandoEnter'];
        }
        if ($tipoVenda == 'M') {
            $text .= 'AMBIENTE: ' . $NMSALA . $printerParams['comandoEnter'];
        }
        if ($tipoVenda == 'B') {
            $text .= 'ORIGEM: VENDA BALCAO' . $printerParams['comandoEnter'];
        }
        $text .= 'OPERADOR: ' . $CDVENDEDOR . ' - ' . $this->util->removeAcentos($NMFANVEN) . $printerParams['comandoEnter'];

        if($toGoString != ''){
            $text .= $toGoString . $printerParams['comandoEnter'];
        }
        $text .= $printerParams['comandoEnter'];
        return $printerParams['comandoEnter'].$text;
    }

    private function printFooter($text, $printerParams, $CDSENHAPED, $NRNOTAFISCALCE) {
        // Monta rodapé.
        $text .= $this->impressaoUtil->centraliza($printerParams, 'NOTA ' . $NRNOTAFISCALCE).$printerParams['comandoEnter'];
        $text .= $this->impressaoUtil->centraliza($printerParams, 'SENHA ' . $CDSENHAPED).$printerParams['comandoEnter'];
        return $text;
    }

    private function buscaNomePorPosicao($produtos, $posicoes){
        if(isset($produtos[0]['CDFILIAL'])) {
            $params = array(
                'CDFILIAL' => $produtos[0]['CDFILIAL'],
                'NRVENDAREST' => $produtos[0]['NRVENDAREST'],
                'NRLUGARMESA' => $posicoes
            );
            $type = array(
                'NRLUGARMESA' => \Doctrine\DBAL\Connection::PARAM_STR_ARRAY
            );
            $nmPosicao = $this->entityManager->getConnection()->fetchAll("BUSCA_NOME_POR_POSICAO", $params, $type);
            return array_column($nmPosicao, null, 'NRLUGARMESA');
        } else {
            return array();
        }

    }

    private function separaProdImpUmPorVez($arrayProdutos){
        $novorray = array();
        foreach ($arrayProdutos as $key => $product) {
            if ($product['IDIMPRODUVEZ'] == 'S' && $product['QTPRODCOMVEN'] > 1 && $product['IDPESAPROD'] == 'N') {
                $quantidade = $product['QTPRODCOMVEN'];
                for ($i=0; $i < $quantidade; $i++){
                    $product['QTPRODCOMVEN']=1;
                    $novorray[]=$product;
                }
            } else {
                $novorray[]=$product;
            }
        }
        return $novorray;
    }

    private function alteraOrdemImpressao($produtos){
        $separados = array();
        $juntos = array();
        foreach ($produtos as $produto){
            if ($produto['IDIMPRODUVEZ'] == 'S'){
                array_push($separados, $produto);
            }
            else {
                array_push($juntos, $produto);
            }
        }

        return array_merge($juntos, $separados);
    }

}
